@@ -29,6 +29,7 @@
 #include <QPainter>
 #include <QMouseEvent>
 #include <QTimer>
+#include <QMimeData>
 
 #ifdef QTONLY
     #include <QMessageBox>
@@ -495,7 +496,7 @@ void VncView::updateImage(int x, int y, int w, int h)
     }
 
     const auto dpr = m_frame.devicePixelRatio();
-    repaint(QRectF(x / dpr * m_horizontalFactor, y / dpr * m_verticalFactor, w / dpr * m_horizontalFactor, h / dpr * m_verticalFactor).toAlignedRect());
+    repaint(QRectF(x / dpr * m_horizontalFactor, y / dpr * m_verticalFactor, w / dpr * m_horizontalFactor, h / dpr * m_verticalFactor).toAlignedRect().adjusted(-1,-1,1,1));
 }
 
 void VncView::setViewOnly(bool viewOnly)
@@ -724,6 +725,14 @@ void VncView::clipboardDataChanged()
     if (m_clipboard->ownsClipboard() || m_dontSendClipboard)
         return;
 
+    if (m_hostPreferences->dontCopyPasswords()) {
+        const QMimeData* data = m_clipboard->mimeData();
+        if (data && data->hasFormat(QLatin1String("x-kde-passwordManagerHint"))) {
+            qCDebug(KRDC) << "VncView::clipboardDataChanged data hasFormat x-kde-passwordManagerHint -- ignoring";
+            return;
+        }
+    }
+
     const QString text = m_clipboard->text(QClipboard::Clipboard);
 
     vncThread.clientCut(text);
